/*
** Copyright (C) 2003-2007 Teus Benschop.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 3 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include "libraries.h"
#include <glib.h>
#include "shell.h"
#include "gwrappers.h"
#include "utilities.h"
#include "subversion.h"
#include "settings.h"
#include "bibledit-subversion.h"
#include "ipc.h"
#include "localizedbooks.h"


Settings * settings;
InterprocessCommunication * ipc;
BookLocalizations * booklocalizations;


int main (int argc, char *argv[])
{
  // Thread support.
  g_thread_init (NULL);
  
  // IPC sytstem.
  InterprocessCommunication myipc (ipcstBibleditSubversion);
  ipc = &myipc;
  
  // Wait a reasonably long time before subversion activity starts.
  // This is because Bibledit itself is starting, and if subversion activity
  // would be starting at the same time too, that would postpone
  // Bibledit getting reasonably responsive.
  g_usleep (5000000);
  
  // Settings object. Take out. Query from bibledit-bin. To be done later on.
  Settings mysettings (false);
  settings = &mysettings;
  // LocalizedBooks object.
  BookLocalizations mybooklocalizations (0);
  booklocalizations = &mybooklocalizations;
  
  // While the database indicates that we ought to run, we keep going.
  while (ipc->get_payload (ipcctSubversionShutdown).empty()) {

    // Variables.
    bool task_was_done = false;
    SubversionTaskType task;
    ustring project;
    unsigned int book, chapter;

    // Get the next available task.
    vector <ustring> available_task;
    available_task = ipc->receive (ipcstBibleditBin, ipcctSubversionJobDescription, available_task);
    if (available_task.size() == 4) {
    
      // Okay, there is something to be done. Pop the job description.
      task = (SubversionTaskType) convert_to_int (available_task[0]);
      project = available_task[1];
      book = convert_to_int (available_task[2]);
      chapter = convert_to_int (available_task[3]);

      // Flag "done" as true, so that only in case of an error, it is set false.
      task_was_done = true;

      // Execute the right task.    
      switch (task) {
        case sttInitializeProject: 
        {
          task_was_done = subversion_execute_initialize_project_local (project);
          break;
        }
        case sttCommitProject:
        { 
          task_was_done = subversion_execute_commit_project (project);
          break;
        }
        case sttStoreChapter: 
        {  
          task_was_done = subversion_execute_store_chapter (project, book, chapter);
          break;
        }
        case sttUpdateProject:
        { 
          task_was_done = subversion_execute_update_project (project);
          break;
        }
        case sttReopenProject:
        { 
          subversion_store_re_open_project (project);
          break;
        }
        case sttOpenChapter:
        {   
          subversion_open_chapter_for_editing (project, book, chapter);
          break;
        }
      }
    
      // Send done or fail.
      vector <ustring> done;
      if (task_was_done) done.push_back (" ");
      ipc->send (ipcstBibleditBin, ipcctSubversionTaskDone, done);   
        
      // There are cases that one task hangs, and nearly blocks the program.
      // Introduce a delay to solve this.
      if (!task_was_done) {
        g_usleep (1000000);
      }
    }
  
    // If the task was done, delay shortly so it can quickly do the next one.
    // Else it delays longer to save system resources.
    if (task_was_done) g_usleep (10000);
    else g_usleep (1000000);

  }
  
  // The job's been done.
  gw_message ("Terminated normally");
  return 0;
}


// Make the executable much smaller by not linking to source of the main executable.
