/*
** Copyright (C) 2003-2007 Teus Benschop.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 3 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include "exception.h"
#include "mainwindow.h"
#include "directories.h"
#include "utilities.h"
#include "gwrappers.h"
#include "directories.h"
#include "dialogshowscript.h"
#include "shell.h"
#include "user.h"
#include "scripting.h"
#include "vacuum.h"
#include "gtkwrappers.h"
#include "upgrade.h"
#include "unixwrappers.h"
#include "settings.h"
#include "constants.h"
#include "ipc.h"
#include "localizedbooks.h"
#include "versifications.h"
#include "mappings.h"
#include "ot-quotations.h"


Settings * settings;
InterprocessCommunication * ipc;
BookLocalizations * booklocalizations;
Versifications * versifications;
Mappings * mappings;


int main (int argc, char *argv[])
{
  // Unhandled exception handler.
  set_terminate (terminator);
  // Handler for unexpected exceptions.
  set_unexpected (my_unexpected);
  // Do not allow to run as root.
  if (runs_as_root ())
    return 1;
  // Check on default data structure
  directories_check_structure ();
  // Settings object. 
  // Only save the settings on exit if there is no scripting.
  Settings mysettings (argc == 1);
  settings = &mysettings;
  // LocalizedBooks object.
  BookLocalizations mybooklocalizations (0);
  booklocalizations = &mybooklocalizations;
  // Versifications object.
  Versifications myversifications (0);
  versifications = &myversifications;
  // Verse mappings object.
  Mappings mymappings (0);
  mappings = &mymappings;
  // Run the backend and terminate if there are commandline arguments
  if (argc > 1) {
    return scripting ((const int) argc, (const char **) argv);
  }
  // We need thread support.
  g_thread_init (NULL);
  // Interprocess Communication object.
  InterprocessCommunication myipc (ipcstBibleditBin);
  ipc = &myipc;
  // Initialize GTK
  gtk_init (&argc, &argv);
  // Wait till vacuum operations from possible previous session are ready.
  vacuum_wait ();
  // If Bibledit is already running, ask what to do.
  if (programs_running_count_basic ("bibledit-bin") > 1) {
    if (gtkw_dialog_question (NULL, "Bibledit is already running.\nWould you like to run another copy?") == GTK_RESPONSE_NO) {
      return 1;
    }    
  }
  // Upgrade data.
  upgrade (true);
  // Start the gui.
  MainWindow mainwindow (0);
  gtk_main ();
  // Quit.
  return 0;
}
