/*
** Copyright (C) 2003-2007 Teus Benschop.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 3 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include <glib.h>
#include "dialogchanges.h"
#include "help.h"
#include "settings.h"
#include "shortcuts.h"
#include "dialogdate.h"
#include "date_time_utils.h"
#include "subversion.h"
#include "utilities.h"
#include "gtkwrappers.h"
#include "gwrappers.h"
#include "unixwrappers.h"
#include "projectutils.h"
#include "compareutils.h"


ChangesDialog::ChangesDialog (References * references)
{
  myreferences = references;
  
  Shortcuts shortcuts (0);

  changesdialog = gtk_dialog_new ();
  gtk_window_set_title (GTK_WINDOW (changesdialog), "Changes");
  gtk_window_set_position (GTK_WINDOW (changesdialog), GTK_WIN_POS_CENTER_ON_PARENT);
  gtk_window_set_type_hint (GTK_WINDOW (changesdialog), GDK_WINDOW_TYPE_HINT_DIALOG);

  dialog_vbox1 = GTK_DIALOG (changesdialog)->vbox;
  gtk_widget_show (dialog_vbox1);

  label2 = gtk_label_new ("This will show the changes that have been made in the project since a certain date and time.");
  gtk_widget_show (label2);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), label2, FALSE, FALSE, 0);
  gtk_label_set_line_wrap (GTK_LABEL (label2), TRUE);
  gtk_misc_set_alignment (GTK_MISC (label2), 0, 0.5);

  hseparator1 = gtk_hseparator_new ();
  gtk_widget_show (hseparator1);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), hseparator1, TRUE, TRUE, 0);

  hbox3 = gtk_hbox_new (FALSE, 4);
  gtk_widget_show (hbox3);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), hbox3, TRUE, TRUE, 0);

  GSList *radiobutton_since_last_group = NULL;
  
  radiobutton_since_last = gtk_radio_button_new_with_mnemonic (NULL, "View changes since last review done on");
  gtk_widget_show (radiobutton_since_last);
  gtk_box_pack_start (GTK_BOX (hbox3), radiobutton_since_last, FALSE, FALSE, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton_since_last), radiobutton_since_last_group);
  radiobutton_since_last_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton_since_last));

  shortcuts.button (radiobutton_since_last);

  label_last_review = gtk_label_new ("");
  gtk_widget_show (label_last_review);
  gtk_box_pack_start (GTK_BOX (hbox3), label_last_review, TRUE, TRUE, 0);
  gtk_misc_set_alignment (GTK_MISC (label_last_review), 0, 0.5);

  /*
  button_last_review = gtk_button_new ();
  gtk_widget_show (button_last_review);
  gtk_box_pack_start (GTK_BOX (hbox3), button_last_review, FALSE, FALSE, 0);

  alignment1 = gtk_alignment_new (0.5, 0.5, 0, 0);
  gtk_widget_show (alignment1);
  gtk_container_add (GTK_CONTAINER (button_last_review), alignment1);

  hbox4 = gtk_hbox_new (FALSE, 2);
  gtk_widget_show (hbox4);
  gtk_container_add (GTK_CONTAINER (alignment1), hbox4);

  image1 = gtk_image_new_from_stock ("gtk-info", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (image1);
  gtk_box_pack_start (GTK_BOX (hbox4), image1, FALSE, FALSE, 0);

  label4 = gtk_label_new_with_mnemonic ("Set");
  gtk_widget_show (label4);
  gtk_box_pack_start (GTK_BOX (hbox4), label4, FALSE, FALSE, 0);

  shortcuts.label (label4);
  */

  hbox5 = gtk_hbox_new (FALSE, 4);
  gtk_widget_show (hbox5);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), hbox5, FALSE, FALSE, 0);

  radiobutton_since_date = gtk_radio_button_new_with_mnemonic (NULL, "View changes since");
  gtk_widget_show (radiobutton_since_date);
  gtk_box_pack_start (GTK_BOX (hbox5), radiobutton_since_date, FALSE, FALSE, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton_since_date), radiobutton_since_last_group);
  radiobutton_since_last_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton_since_date));

  shortcuts.button (radiobutton_since_date);

  label_since = gtk_label_new ("");
  gtk_widget_show (label_since);
  gtk_box_pack_start (GTK_BOX (hbox5), label_since, TRUE, TRUE, 0);
  gtk_misc_set_alignment (GTK_MISC (label_since), 0, 0.5);

  button_since = gtk_button_new ();
  gtk_widget_show (button_since);
  gtk_box_pack_start (GTK_BOX (hbox5), button_since, FALSE, FALSE, 0);

  alignment2 = gtk_alignment_new (0.5, 0.5, 0, 0);
  gtk_widget_show (alignment2);
  gtk_container_add (GTK_CONTAINER (button_since), alignment2);

  hbox6 = gtk_hbox_new (FALSE, 2);
  gtk_widget_show (hbox6);
  gtk_container_add (GTK_CONTAINER (alignment2), hbox6);

  image2 = gtk_image_new_from_stock ("gtk-info", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (image2);
  gtk_box_pack_start (GTK_BOX (hbox6), image2, FALSE, FALSE, 0);

  label6 = gtk_label_new_with_mnemonic ("Set");
  gtk_widget_show (label6);
  gtk_box_pack_start (GTK_BOX (hbox6), label6, FALSE, FALSE, 0);

  shortcuts.label (label6);

  dialog_action_area1 = GTK_DIALOG (changesdialog)->action_area;
  gtk_widget_show (dialog_action_area1);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area1), GTK_BUTTONBOX_END);

  new InDialogHelp (changesdialog, &shortcuts, "changes");
  
  cancelbutton1 = gtk_button_new_from_stock ("gtk-cancel");
  gtk_widget_show (cancelbutton1);
  gtk_dialog_add_action_widget (GTK_DIALOG (changesdialog), cancelbutton1, GTK_RESPONSE_CANCEL);
  GTK_WIDGET_SET_FLAGS (cancelbutton1, GTK_CAN_DEFAULT);

  shortcuts.stockbutton (cancelbutton1);

  okbutton1 = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show (okbutton1);
  gtk_dialog_add_action_widget (GTK_DIALOG (changesdialog), okbutton1, GTK_RESPONSE_OK);
  GTK_WIDGET_SET_FLAGS (okbutton1, GTK_CAN_DEFAULT);

  shortcuts.stockbutton (okbutton1);
  
  shortcuts.process();

  g_signal_connect ((gpointer) radiobutton_since_last, "toggled", G_CALLBACK (on_radiobutton_since_toggled), gpointer(this));
  g_signal_connect ((gpointer) radiobutton_since_date, "toggled", G_CALLBACK (on_radiobutton_since_toggled), gpointer(this));
  //g_signal_connect ((gpointer) button_last_review, "clicked", G_CALLBACK (on_button_last_review_clicked), gpointer(this));
  g_signal_connect ((gpointer) button_since, "clicked", G_CALLBACK (on_button_since_clicked), gpointer(this));
  g_signal_connect ((gpointer) okbutton1, "clicked", G_CALLBACK (on_okbutton1_clicked), gpointer(this));

  gtk_widget_grab_focus (okbutton1);
  gtk_widget_grab_default (okbutton1);
  
  gui ();
}


ChangesDialog::~ChangesDialog ()
{
  // Delete project that has the history.
  if (!history_project.empty()) {
    project_delete (history_project);
  }
  // Destroy the dialog.
  gtk_widget_destroy (changesdialog);
}


int ChangesDialog::run ()
{
  return gtk_dialog_run (GTK_DIALOG (changesdialog));
}



void ChangesDialog::on_radiobutton_since_toggled (GtkToggleButton *togglebutton, gpointer user_data)
{
  ((ChangesDialog *) user_data)->gui ();
}


/*
void ChangesDialog::on_button_last_review_clicked (GtkButton *button, gpointer user_data)
{
  ((ChangesDialog *) user_data)->on_button_last_review ();
}


void ChangesDialog::on_button_last_review ()
{
  ProjectConfiguration * projectconfig = settings->projectconfig ("");
  guint32 seconds = projectconfig->changes_last_review ();
  DateDialog dialog (&seconds, true);
  if (dialog.run () == GTK_RESPONSE_OK) {
    projectconfig->changes_last_review_set (seconds);
    gui ();
  }
}
*/


void ChangesDialog::on_button_since_clicked (GtkButton *button, gpointer user_data)
{
  ((ChangesDialog *) user_data)->on_button_since ();
}


void ChangesDialog::on_button_since ()
{
  extern Settings * settings;
  ProjectConfiguration * projectconfig = settings->projectconfig (settings->genconfig.project_get());
  guint32 seconds = projectconfig->changes_since_get ();
  DateDialog dialog (&seconds, true);
  if (dialog.run () == GTK_RESPONSE_OK) {
    projectconfig->changes_since_set (seconds);
    gui ();
  }
}


void ChangesDialog::on_okbutton1_clicked (GtkButton *button, gpointer user_data)
{
  ((ChangesDialog *) user_data)->on_okbutton ();
}


void ChangesDialog::on_okbutton ()
{
  // Configuration.
  extern Settings * settings;
  ProjectConfiguration * projectconfig = settings->projectconfig (settings->genconfig.project_get());
  
  // Whether viewing changes since the last review done.
  bool changes_since_last_review = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (radiobutton_since_last));

  // Retrieve the seconds to check the changes since.
  unsigned int seconds = projectconfig->changes_since_get ();
  if (changes_since_last_review)
    seconds = projectconfig->changes_last_review_get ();
  
  // Checkout the history project.
  // This contains the data of our project as it was at the date/time given.
  history_project = settings->genconfig.project_get() + "_as_it_was_at_" + convert_to_string (seconds);
  ustring history_project_data_directory = project_data_directory_project (history_project);
  gw_mkdir_with_parents (history_project_data_directory);
  {
    ustring workingdirectory = gw_path_get_dirname (history_project_data_directory);
    ustring repositorypath = subversion_project_get_url (settings->genconfig.project_get());
    ustring checkoutdirectory = gw_build_filename (workingdirectory, gw_path_get_basename (repositorypath));
    GwSpawn spawn ("svn");
    spawn.workingdirectory (workingdirectory);
    spawn.arg ("checkout");
    spawn.arg (repositorypath);
    spawn.arg ("--revision");
    spawn.arg (subversion_revision_date_time (seconds));
    spawn.arg ("--non-interactive");
    spawn.progress ("Retrieving data from history", false);
    spawn.run ();
    if (spawn.exitstatus == 0) {
      unix_rmdir (history_project_data_directory);
      unix_mv (checkoutdirectory, history_project_data_directory);
    } else {
      gtkw_dialog_error (changesdialog, "Failed to retrieve history");
      return;
    }
  }

  // Run comparison.
  compare_with (myreferences, settings->genconfig.project_get(), history_project, true);
  
  // If reviewing changes since last review, set the date for the next review.  
  if (changes_since_last_review) {
    projectconfig->changes_last_review_set (date_time_seconds_get_current ());    
  }  
}


void ChangesDialog::gui ()
{
  // Configurations.
  extern Settings * settings;
  ProjectConfiguration * projectconfig = settings->projectconfig (settings->genconfig.project_get());
  // Seconds.
  unsigned int seconds;
  // Fetch seconds since last review. 
  // If that is zero, that means, uninitialized, take the date/time from
  // the first revision in the subversion repository.
  seconds = projectconfig->changes_last_review_get ();
  if (seconds == 0) {
    seconds = subversion_log_date_time_at_revision (settings->genconfig.project_get(), 1);
    seconds--;
    projectconfig->changes_last_review_set (seconds);
  }
  // Set label to the human readable date and time.
  ustring s;
  s = date_time_seconds_human_readable (seconds, true);
  gtk_label_set_text (GTK_LABEL (label_last_review), s.c_str());
  // Same story for the seconds to view the differences since.
  seconds = projectconfig->changes_since_get ();
  if (seconds == 0) {
    seconds = subversion_log_date_time_at_revision (settings->genconfig.project_get(), 1);
    seconds--;
    projectconfig->changes_since_set (seconds);
  }
  s = date_time_seconds_human_readable (seconds, true);
  gtk_label_set_text (GTK_LABEL (label_since), s.c_str());
  // Sensitivity of buttons.
  bool changes_since_last_review = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (radiobutton_since_last));
  gtk_widget_set_sensitive (button_since, !changes_since_last_review);
}
