/*
** Copyright (C) 2003-2007 Teus Benschop.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 3 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include <glib.h>
#include "dialogresourceconverter.h"
#include "help.h"
#include "gtkwrappers.h"
#include "utilities.h"
#include "progresswindow.h"
#include "shortcuts.h"
#include "unixwrappers.h"
#include "bible.h"
#include "books.h"
#include "html.h"
#include "gui.h"
#include "dialogreviewanchors.h"
#include "resource_viewer.h"


ResourceConverterDialog::ResourceConverterDialog (int dummy)
{
  // Initialize variables.
  currentbook = 0;
  currentchapter = 0;
  examined = false;
  reviewed = false;
  written = false;
  
  Shortcuts shortcuts (0);

  resourceconverterdialog = gtk_dialog_new ();
  gtk_window_set_title (GTK_WINDOW (resourceconverterdialog), "Resource Converter");
  gtk_window_set_position (GTK_WINDOW (resourceconverterdialog), GTK_WIN_POS_CENTER_ON_PARENT);
  gtk_window_set_modal (GTK_WINDOW (resourceconverterdialog), TRUE);

  dialog_vbox1 = GTK_DIALOG (resourceconverterdialog)->vbox;
  gtk_widget_show (dialog_vbox1);

  label1 = gtk_label_new ("This tool will make a web page fit for the Bibledit Resource Viewer.\nIt does that by inserting an appropriate anchor at the start of each verse it finds.");
  gtk_widget_show (label1);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), label1, FALSE, FALSE, 0);
  gtk_misc_set_alignment (GTK_MISC (label1), 0, 0.5);

  hseparator1 = gtk_hseparator_new ();
  gtk_widget_show (hseparator1);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), hseparator1, TRUE, TRUE, 0);

  hbox3 = gtk_hbox_new (FALSE, 6);
  gtk_widget_show (hbox3);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), hbox3, TRUE, TRUE, 0);

  image_open_ok = gtk_image_new_from_stock ("gtk-apply", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (image_open_ok);
  gtk_box_pack_start (GTK_BOX (hbox3), image_open_ok, FALSE, FALSE, 0);

  label_open_ok = gtk_label_new ("Done:");
  gtk_widget_show (label_open_ok);
  gtk_box_pack_start (GTK_BOX (hbox3), label_open_ok, FALSE, FALSE, 0);

  label_open_action = gtk_label_new ("Open a file that has a web page to be converted");
  gtk_widget_show (label_open_action);
  gtk_box_pack_start (GTK_BOX (hbox3), label_open_action, TRUE, TRUE, 0);
  gtk_misc_set_alignment (GTK_MISC (label_open_action), 0, 0.5);

  button_open = gtk_button_new ();
  gtk_widget_show (button_open);
  gtk_box_pack_start (GTK_BOX (hbox3), button_open, FALSE, FALSE, 0);

  alignment1 = gtk_alignment_new (0.5, 0.5, 0, 0);
  gtk_widget_show (alignment1);
  gtk_container_add (GTK_CONTAINER (button_open), alignment1);

  hbox2 = gtk_hbox_new (FALSE, 2);
  gtk_widget_show (hbox2);
  gtk_container_add (GTK_CONTAINER (alignment1), hbox2);

  image1 = gtk_image_new_from_stock ("gtk-open", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (image1);
  gtk_box_pack_start (GTK_BOX (hbox2), image1, FALSE, FALSE, 0);

  label3 = gtk_label_new_with_mnemonic ("Open");
  gtk_widget_show (label3);
  gtk_box_pack_start (GTK_BOX (hbox2), label3, FALSE, FALSE, 0);

  shortcuts.label (label3);
  
  label_open_filename = gtk_label_new ("");
  gtk_widget_show (label_open_filename);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), label_open_filename, FALSE, FALSE, 0);
  gtk_misc_set_alignment (GTK_MISC (label_open_filename), 0, 0.5);

  hseparator2 = gtk_hseparator_new ();
  gtk_widget_show (hseparator2);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), hseparator2, TRUE, TRUE, 0);

  label2 = gtk_label_new ("Open the file in a web browser to have a close look at it\nand to find out how the tool could recognize books, chapters and verses.\nThe tool recognizes a verse by its signature. Select which signature to look for:");
  gtk_widget_show (label2);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), label2, FALSE, FALSE, 0);
  gtk_misc_set_alignment (GTK_MISC (label2), 0, 0.5);

  GSList *radiobutton_signature_book_space_chapter_colon_verse_line_start_group = NULL;

  radiobutton_signature_book_space_chapter_colon_verse_line_start = gtk_radio_button_new_with_mnemonic (NULL, "Bookname, space, chapter, colon or dot, verse at the beginning of a line");
  gtk_widget_show (radiobutton_signature_book_space_chapter_colon_verse_line_start);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), radiobutton_signature_book_space_chapter_colon_verse_line_start, FALSE, FALSE, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton_signature_book_space_chapter_colon_verse_line_start), radiobutton_signature_book_space_chapter_colon_verse_line_start_group);
  radiobutton_signature_book_space_chapter_colon_verse_line_start_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton_signature_book_space_chapter_colon_verse_line_start));

  shortcuts.button (radiobutton_signature_book_space_chapter_colon_verse_line_start);

  radiobutton_signature_book_and_chapter_on_a_line_verses_follow = gtk_radio_button_new_with_mnemonic (NULL, "Bookname and chapter on one line, followed by the verse numbers each starting a line");
  gtk_widget_show (radiobutton_signature_book_and_chapter_on_a_line_verses_follow);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), radiobutton_signature_book_and_chapter_on_a_line_verses_follow, FALSE, FALSE, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton_signature_book_and_chapter_on_a_line_verses_follow), radiobutton_signature_book_space_chapter_colon_verse_line_start_group);
  radiobutton_signature_book_space_chapter_colon_verse_line_start_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton_signature_book_space_chapter_colon_verse_line_start));

  shortcuts.button (radiobutton_signature_book_and_chapter_on_a_line_verses_follow);

  hseparator3 = gtk_hseparator_new ();
  gtk_widget_show (hseparator3);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), hseparator3, TRUE, TRUE, 0);

  hbox4 = gtk_hbox_new (FALSE, 6);
  gtk_widget_show (hbox4);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), hbox4, TRUE, TRUE, 0);

  image_examine_ok = gtk_image_new_from_stock ("gtk-apply", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (image_examine_ok);
  gtk_box_pack_start (GTK_BOX (hbox4), image_examine_ok, FALSE, FALSE, 0);

  label_examine_ok = gtk_label_new ("Done:");
  gtk_widget_show (label_examine_ok);
  gtk_box_pack_start (GTK_BOX (hbox4), label_examine_ok, FALSE, FALSE, 0);

  label_examine_action = gtk_label_new ("Examine the file for possible placement of anchors");
  gtk_widget_show (label_examine_action);
  gtk_box_pack_start (GTK_BOX (hbox4), label_examine_action, TRUE, TRUE, 0);
  gtk_misc_set_alignment (GTK_MISC (label_examine_action), 0, 0.5);

  button_examine = gtk_button_new ();
  gtk_widget_show (button_examine);
  gtk_box_pack_start (GTK_BOX (hbox4), button_examine, FALSE, FALSE, 0);

  alignment2 = gtk_alignment_new (0.5, 0.5, 0, 0);
  gtk_widget_show (alignment2);
  gtk_container_add (GTK_CONTAINER (button_examine), alignment2);

  hbox5 = gtk_hbox_new (FALSE, 2);
  gtk_widget_show (hbox5);
  gtk_container_add (GTK_CONTAINER (alignment2), hbox5);

  image5 = gtk_image_new_from_stock ("gtk-zoom-fit", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (image5);
  gtk_box_pack_start (GTK_BOX (hbox5), image5, FALSE, FALSE, 0);

  label9 = gtk_label_new_with_mnemonic ("Examine");
  gtk_widget_show (label9);
  gtk_box_pack_start (GTK_BOX (hbox5), label9, FALSE, FALSE, 0);

  shortcuts.label (label9);
  
  label_examine_results = gtk_label_new ("");
  gtk_widget_show (label_examine_results);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), label_examine_results, FALSE, FALSE, 0);
  gtk_misc_set_alignment (GTK_MISC (label_examine_results), 0, 0.5);

  hseparator4 = gtk_hseparator_new ();
  gtk_widget_show (hseparator4);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), hseparator4, TRUE, TRUE, 0);

  hbox6 = gtk_hbox_new (FALSE, 6);
  gtk_widget_show (hbox6);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), hbox6, TRUE, TRUE, 0);

  image_review_ok = gtk_image_new_from_stock ("gtk-apply", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (image_review_ok);
  gtk_box_pack_start (GTK_BOX (hbox6), image_review_ok, FALSE, FALSE, 0);

  label_review_ok = gtk_label_new ("Done:");
  gtk_widget_show (label_review_ok);
  gtk_box_pack_start (GTK_BOX (hbox6), label_review_ok, FALSE, FALSE, 0);

  label_review_action = gtk_label_new ("Review the anchors that will be placed");
  gtk_widget_show (label_review_action);
  gtk_box_pack_start (GTK_BOX (hbox6), label_review_action, TRUE, TRUE, 0);
  gtk_misc_set_alignment (GTK_MISC (label_review_action), 0, 0.5);

  button_review = gtk_button_new ();
  gtk_widget_show (button_review);
  gtk_box_pack_start (GTK_BOX (hbox6), button_review, FALSE, FALSE, 0);

  alignment3 = gtk_alignment_new (0.5, 0.5, 0, 0);
  gtk_widget_show (alignment3);
  gtk_container_add (GTK_CONTAINER (button_review), alignment3);

  hbox7 = gtk_hbox_new (FALSE, 2);
  gtk_widget_show (hbox7);
  gtk_container_add (GTK_CONTAINER (alignment3), hbox7);

  image7 = gtk_image_new_from_stock ("gtk-index", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (image7);
  gtk_box_pack_start (GTK_BOX (hbox7), image7, FALSE, FALSE, 0);

  label13 = gtk_label_new_with_mnemonic ("Review");
  gtk_widget_show (label13);
  gtk_box_pack_start (GTK_BOX (hbox7), label13, FALSE, FALSE, 0);

  shortcuts.label (label13);  

  label_review_result = gtk_label_new ("");
  gtk_widget_show (label_review_result);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), label_review_result, FALSE, FALSE, 0);
  gtk_misc_set_alignment (GTK_MISC (label_review_result), 0, 0.5);

  hseparator5 = gtk_hseparator_new ();
  gtk_widget_show (hseparator5);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), hseparator5, TRUE, TRUE, 0);

  hbox8 = gtk_hbox_new (FALSE, 6);
  gtk_widget_show (hbox8);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), hbox8, TRUE, TRUE, 0);

  image_write_ok = gtk_image_new_from_stock ("gtk-apply", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (image_write_ok);
  gtk_box_pack_start (GTK_BOX (hbox8), image_write_ok, FALSE, FALSE, 0);

  label_write_ok = gtk_label_new ("Done:");
  gtk_widget_show (label_write_ok);
  gtk_box_pack_start (GTK_BOX (hbox8), label_write_ok, FALSE, FALSE, 0);

  label_write_action = gtk_label_new ("write the anchors to the file");
  gtk_widget_show (label_write_action);
  gtk_box_pack_start (GTK_BOX (hbox8), label_write_action, TRUE, TRUE, 0);
  gtk_misc_set_alignment (GTK_MISC (label_write_action), 0, 0.5);

  button_write = gtk_button_new ();
  gtk_widget_show (button_write);
  gtk_box_pack_start (GTK_BOX (hbox8), button_write, FALSE, FALSE, 0);

  alignment4 = gtk_alignment_new (0.5, 0.5, 0, 0);
  gtk_widget_show (alignment4);
  gtk_container_add (GTK_CONTAINER (button_write), alignment4);

  hbox9 = gtk_hbox_new (FALSE, 2);
  gtk_widget_show (hbox9);
  gtk_container_add (GTK_CONTAINER (alignment4), hbox9);

  image9 = gtk_image_new_from_stock ("gtk-save", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (image9);
  gtk_box_pack_start (GTK_BOX (hbox9), image9, FALSE, FALSE, 0);

  label17 = gtk_label_new_with_mnemonic ("Write");
  gtk_widget_show (label17);
  gtk_box_pack_start (GTK_BOX (hbox9), label17, FALSE, FALSE, 0);
  
  shortcuts.label (label17);

  label_write_result = gtk_label_new ("anchors were succesfully written");
  gtk_widget_show (label_write_result);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), label_write_result, FALSE, FALSE, 0);
  gtk_misc_set_alignment (GTK_MISC (label_write_result), 0, 0.5);

  dialog_action_area1 = GTK_DIALOG (resourceconverterdialog)->action_area;
  gtk_widget_show (dialog_action_area1);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area1), GTK_BUTTONBOX_END);

  new InDialogHelp (resourceconverterdialog, &shortcuts, "resource_converter");

  cancelbutton = gtk_button_new_from_stock ("gtk-cancel");
  gtk_widget_show (cancelbutton);
  gtk_dialog_add_action_widget (GTK_DIALOG (resourceconverterdialog), cancelbutton, GTK_RESPONSE_CANCEL);
  GTK_WIDGET_SET_FLAGS (cancelbutton, GTK_CAN_DEFAULT);

  okbutton = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show (okbutton);
  gtk_dialog_add_action_widget (GTK_DIALOG (resourceconverterdialog), okbutton, GTK_RESPONSE_OK);
  GTK_WIDGET_SET_FLAGS (okbutton, GTK_CAN_DEFAULT);

  shortcuts.stockbutton (cancelbutton);
  shortcuts.stockbutton (okbutton);
  shortcuts.process ();
  
  gtk_widget_set_sensitive (okbutton, false);

  g_signal_connect ((gpointer) button_open, "clicked", G_CALLBACK (on_button_open_clicked), gpointer (this));
  g_signal_connect ((gpointer) button_examine, "clicked", G_CALLBACK (on_button_examine_clicked), gpointer (this));
  g_signal_connect ((gpointer) button_review, "clicked", G_CALLBACK (on_button_review_clicked), gpointer (this));
  g_signal_connect ((gpointer) button_write, "clicked", G_CALLBACK (on_button_write_clicked), gpointer (this));
  g_signal_connect ((gpointer) okbutton, "clicked", G_CALLBACK (on_okbutton_clicked), gpointer (this));

  gtk_widget_grab_focus (okbutton);
  gtk_widget_grab_default (okbutton);
  
  gui ();
}


ResourceConverterDialog::~ResourceConverterDialog ()
{
  gtk_widget_destroy (resourceconverterdialog);
}


int ResourceConverterDialog::run ()
{
  return gtk_dialog_run (GTK_DIALOG (resourceconverterdialog));
}


void ResourceConverterDialog::on_button_open_clicked (GtkButton *button, gpointer user_data)
{
  ((ResourceConverterDialog *) user_data)->on_openbutton ();
}


void ResourceConverterDialog::on_openbutton ()
{
  // Open a file.
  ustring s = gtkw_file_chooser_open (resourceconverterdialog, "", filename);
  if (!s.empty()) {
    // Store filename, read it.
    filename = s;
    ReadText rt (filename, true, false);
    lines = rt.lines;    
    // Clear flag for next step.
    examined = false;
    // Set gui.
    s.append (": ");
    s.append (convert_to_string (lines.size()));
    s.append (" lines");
    gtk_label_set_text (GTK_LABEL (label_open_filename), s.c_str());
    gtk_widget_grab_focus (radiobutton_signature_book_space_chapter_colon_verse_line_start);
    gui ();
  }
}


void ResourceConverterDialog::on_button_examine_clicked (GtkButton *button, gpointer user_data)
{
  ((ResourceConverterDialog *) user_data)->on_examinebutton ();
}


void ResourceConverterDialog::on_examinebutton ()
{
  // Clear any previous anchors.
  anchors_books.clear();
  anchors_chapters.clear();
  anchors_verses.clear();
  anchors_positions.clear ();
  currentbook = 0;
  currentchapter = 0;
  
  // Go through all the lines
  // Show progress. Allow cancel.
  ProgressWindow progress ("Examining data", true);
  progress.set_iterate (0, 1, lines.size());
  for (unsigned int i = 0; i < lines.size(); i++) {
    progress.iterate ();
    if (progress.cancel) return;
      
    // Produce a line to work on.
    // If the next line does not start with a "<", add it to the main one.
    // This is because at times the html editor may cut a signature in the middle.
    // Making one clean line also greatly reduces the amount of spurious 
    // anchors that would otherwise be found.
    ustring line (lines[i]);
    while (   (i + 1 != lines.size()) 
           && (!lines[i + 1].empty()) 
           && (lines[i + 1].substr (0, 1) != "<")) {
      i++;
      progress.iterate ();
      line.append (" ");
      line.append (lines[i]);
    }
    
    // Produce a clean line, without the html code, just the pure text only.
    line = html_remove_code_between_less_than_and_greater_than_signs (line);
    // Replace the long with the short hyphen. Important when recognizing
    // ranges of verses.
    replace_text (line, "–", "-");    

    // If there is no text, skip rest.
    if (line.empty ()) continue;
    
    // Do the appropriate examination.
    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (radiobutton_signature_book_space_chapter_colon_verse_line_start)))
      examine_book_space_chapter_colon_verse_line_start (i, line);
    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (radiobutton_signature_book_and_chapter_on_a_line_verses_follow)))
      examine_book_and_chapter_on_a_line_verses_follow (i, line);
  }

  // Set gui.
  ustring label ("Possible locations for anchors found: " + convert_to_string (anchors_books.size()));
  gtk_label_set_text (GTK_LABEL (label_examine_results), label.c_str());
  examined = true;  
  reviewed = false;
  gui ();
  gtk_widget_grab_focus (button_review);
}


void ResourceConverterDialog::on_button_review_clicked (GtkButton *button, gpointer user_data)
{
  ((ResourceConverterDialog *) user_data)->on_reviewbutton ();
}


void ResourceConverterDialog::on_reviewbutton ()
{
  ReviewAnchorsDialog dialog (&anchors_books, &anchors_chapters, &anchors_verses, &anchors_positions);
  if (dialog.run () == GTK_RESPONSE_OK) {
    ustring label ("Anchors that will be placed: " + convert_to_string (anchors_books.size()));
    gtk_label_set_text (GTK_LABEL (label_review_result), label.c_str());
    reviewed = true;
    written = false;
    gui ();
    gtk_widget_grab_focus (button_write);
  }
}


void ResourceConverterDialog::on_button_write_clicked (GtkButton *button, gpointer user_data)
{
  ((ResourceConverterDialog *) user_data)->on_writebutton ();
}


void ResourceConverterDialog::on_writebutton ()
{
  // Ask whether to write the new data. If not, bail out.
  ustring question;
  question.append ("Would you like to write the anchors to the file?");
  question.append ("\nWarning: This will overwrite the original file.");
  if (gtkw_dialog_question (resourceconverterdialog, question) != GTK_RESPONSE_YES) return;

  // Insert the anchors.
  for (unsigned int i = 0; i < anchors_positions.size(); i++) {
    insert_anchor (lines[anchors_positions[i]], html_create_anchor (resource_viewer_produce_anchor (anchors_books[i], anchors_chapters[i], anchors_verses[i]), ""));
  }

  // Write the data.
  write_lines (filename, lines);
  
  // GUI.
  gtk_label_set_text (GTK_LABEL (label_write_result), "The anchors were written");
  written = true;
  gui ();
  gtk_widget_grab_focus (okbutton);
  
}


void ResourceConverterDialog::on_okbutton_clicked (GtkButton *button, gpointer user_data)
{
  ((ResourceConverterDialog *) user_data)->on_okbutton ();
}


void ResourceConverterDialog::on_okbutton ()
{
}


void ResourceConverterDialog::gui ()
{
  // The file, whether loaded.
  bool filename_ok = !filename.empty();
  if (lines.empty()) filename_ok = false;
  gui_okay (image_open_ok, label_open_ok, filename_ok);

  // If the file has not yet been examined, clear the anchors.
  if (!examined) {
    anchors_books.clear();
    anchors_chapters.clear();
    anchors_verses.clear();
    anchors_positions.clear();
    currentbook = 0;
    currentchapter = 0;
  }
  
  // We can examine the file if we've got data.
  gtk_widget_set_sensitive (button_examine, !lines.empty());
  
  // Whether file was examined.
  bool examined_ok =  filename_ok && examined;
  gui_okay (image_examine_ok, label_examine_ok, examined_ok);
  if (!examined) gtk_label_set_text (GTK_LABEL (label_examine_results), "");

  // We can review the file if it has been examined.
  gtk_widget_set_sensitive (button_review, examined);
  
  // Whether file was reviewed.
  bool reviewed_ok = filename_ok && examined_ok && reviewed;
  gui_okay (image_review_ok, label_review_ok, reviewed_ok);
  if (!reviewed) gtk_label_set_text (GTK_LABEL (label_review_result), "");
  
  // We can save the file if it has been reviewed.
  gtk_widget_set_sensitive (button_write, reviewed_ok);
  
  // Whether the file was written.
  bool written_ok = filename_ok && examined_ok && reviewed_ok && written;
  gui_okay (image_write_ok, label_write_ok, written_ok);
  
  // If file wan't written, clear label.
  if (!written_ok) gtk_label_set_text (GTK_LABEL (label_write_result), "");

  // OK is sensitive if everything is done.
  gtk_widget_set_sensitive (okbutton, written_ok);
}


void ResourceConverterDialog::examine_book_space_chapter_colon_verse_line_start (unsigned int linenumber, const ustring& line)
{
  // Parse the line into words separated by spaces,
  // and go through the second, third, and fourth word.
  Parse parse (line, false);
  unsigned int fourth_word = CLAMP (4, 0, parse.words.size());
  for (unsigned int i = 1; i < fourth_word; i++) {
    // Look for the "<chapter>:<verse>".
    if (unix_fnmatch ("*[0-9][:.][0-9]*", parse.words[i])) {

      // Find the position of this chapter:verse in the line.
      size_t chapter_verse_pos = line.find (parse.words[i]);
      
      // See if a sensible reference can be found in the raw text, from the
      // beginning of the line.
      ustring rawbook = line.substr (0, chapter_verse_pos - 1);
      unsigned int book = book_find_valid (rawbook);
      if (book) {
        unsigned int chapter;
        ustring verse;
        if (reference_discover (book, 0, "", parse.words[i], book, chapter, verse)) {
          // Store the anchor(s).
          // We put "anchors" in a possible plural, because ranges and sequences
          // are covered too.
          vector <unsigned int> verses = verse_range_sequence (verse);
          for (unsigned int i2 = 0; i2 < verses.size(); i2++) {
            anchors_books.push_back (book);
            anchors_chapters.push_back (chapter);
            anchors_verses.push_back (verses[i2]);
            anchors_positions.push_back (linenumber);
          }
          break;
        }
      }
    }    
  }
}


void ResourceConverterDialog::examine_book_and_chapter_on_a_line_verses_follow (unsigned int linenumber, const ustring& line)
{
  // See if the whole line is a book/chapter.
  // If so, store them, and bail out.
  unsigned int book;
  unsigned int chapter;
  ustring verse;
  if (line.length() < 25) {
    if (!number_in_string (line).empty()) {
      if (reference_discover (0, 0, "", line, book, chapter, verse)) {
        if (book && chapter) {
          currentbook = book;
          currentchapter = chapter;
          return;
        } 
      }
    }
  }

  // If there is no book or chapter yet, bail out.
  if (!currentbook) return;
  if (!currentchapter) return;
  
  // Check if a line starts with a verse number. If not, bail out.
  verse = number_in_string (line);
  if (verse.empty()) return;
  if (line.find (verse) != 0) return;
  
  // Store the anchor.
  anchors_books.push_back (currentbook);
  anchors_chapters.push_back (currentchapter);
  anchors_verses.push_back (convert_to_int (verse));
  anchors_positions.push_back (linenumber);
}


void ResourceConverterDialog::insert_anchor (ustring& line, const ustring& anchor)
// Inserts the anchor in the line.
{
  // Produce the anchor to insert.
  ustring anchor2 (html_create_anchor (anchor, ""));
  // If there is a ">" in the line, insert the anchor straight after it.
  // Else insert it at the beginning of the line.
  size_t pos = line.find (">");
  if (pos == string::npos) pos = 0;
  else pos++;
  line.insert (pos, anchor);
}
