/*
** Copyright (C) 2003-2007 Teus Benschop.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 3 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include "htmlbrowser.h"
#include "help.h"
extern "C" {
  #include <gtkhtml/gtkhtml.h>
}
#include "directories.h"
#include "gwrappers.h"
#include <libxml/nanohttp.h>



void help_open (GtkButton * button, gpointer user_data)
{
  gchar * file = (gchar *) user_data;
  ustring url = "localhost:51516/olh_";
  url.append (file);
  url.append (".html");
  htmlbrowser (url, true); 
}


InDialogHelp::InDialogHelp (GtkWidget * dialog, Shortcuts * shortcuts, gchar * topic)
{
  // Save and initialize variables.
  mydialog = dialog;
  mytopic = topic;
  
  // If no help is given, take a default one.
  if (!mytopic) mytopic = "none";

  helpbutton = gtk_toggle_button_new ();
  gtk_widget_show (helpbutton);
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area), helpbutton, FALSE, FALSE, 0);
  GTK_WIDGET_SET_FLAGS (helpbutton, GTK_CAN_DEFAULT);

  alignment = gtk_alignment_new (0.5, 0.5, 0, 0);
  gtk_widget_show (alignment);
  gtk_container_add (GTK_CONTAINER (helpbutton), alignment);

  hbox = gtk_hbox_new (FALSE, 2);
  gtk_widget_show (hbox);
  gtk_container_add (GTK_CONTAINER (alignment), hbox);

  image = gtk_image_new_from_stock ("gtk-help", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (image);
  gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 0);

  label = gtk_label_new_with_mnemonic ("_Help");
  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
  
  if (shortcuts) shortcuts->label (label);
    
  g_signal_connect ((gpointer) helpbutton, "clicked", G_CALLBACK (on_helpbutton_activated), gpointer (this));  
  g_signal_connect ((gpointer) dialog, "delete_event", G_CALLBACK (on_dialog_delete_event), gpointer (this));
}


InDialogHelp::~InDialogHelp ()
{
}


void InDialogHelp::on_helpbutton_activated (GtkButton * button, gpointer user_data)
{
  ((InDialogHelp *) user_data)->on_helpbutton();  
}


void InDialogHelp::on_helpbutton ()
{
  bool button_on = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (helpbutton));
  if (button_on) {

    // Create gui.
    scrolledwindow = gtk_scrolled_window_new (NULL, NULL);
    gtk_widget_show (scrolledwindow);
    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (mydialog)->vbox), scrolledwindow, TRUE, TRUE, 0);
    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

    htmlview = gtk_html_new ();
    gtk_widget_show (htmlview);
    gtk_container_add (GTK_CONTAINER (scrolledwindow), htmlview);
 
    // Base to work against.
    gtk_html_set_base (GTK_HTML (htmlview), "http://localhost:51516");

    // Assemble the url to load.    
    ustring url = "http://localhost:51516/olh_";
    url.append (mytopic);
    url.append (".html");
 
    // Fetch and store this url.    
    ustring htmloutput = gw_build_filename (directories_get_temp (), "indialoghelp.html");
    xmlNanoHTTPFetch (url.c_str(), htmloutput.c_str(), NULL);
    xmlNanoHTTPCleanup();

    // Proceed with html widget.
    gtk_html_set_editable (GTK_HTML (htmlview), FALSE);
    gtk_html_allow_selection (GTK_HTML (htmlview), TRUE);
    g_signal_connect (G_OBJECT (htmlview), "url-requested", G_CALLBACK (on_html_url_requested), gpointer (this));
    g_signal_connect (G_OBJECT (htmlview), "link-clicked", G_CALLBACK (on_html_link_clicked), gpointer (this));    
    
    // Load the file into memory.    
    gchar *contents;
    g_file_get_contents (htmloutput.c_str(), &contents, NULL, NULL);

    // Write the contents to the widget.
    GtkHTMLStream *stream;
    stream = gtk_html_begin (GTK_HTML(htmlview));
    if (contents) gtk_html_write (GTK_HTML(htmlview), stream, contents, -1);
    gtk_html_end (GTK_HTML(htmlview), stream, GTK_HTML_STREAM_OK);

    // Clean up.   
    g_free (contents);
    unlink (htmloutput.c_str());
    
  } else {
    gtk_widget_destroy (scrolledwindow);
  }
}


gboolean InDialogHelp::on_dialog_delete_event (GtkWidget *widget, GdkEvent *event, gpointer user_data)
{
  // If the dialog gets deleted, then destroy the InDialogHelp object.
  InDialogHelp * this_is_me = (InDialogHelp *) user_data;
  delete this_is_me;
  return false;
}


gboolean InDialogHelp::on_html_url_requested (GtkWidget * html, const gchar * url, gpointer stream, gpointer user_data)
{
  return true;
}


gboolean InDialogHelp::on_html_link_clicked (GtkWidget * html, const gchar * url, gpointer user_data)
{
  // Call handler.
  ((InDialogHelp *) user_data)->html_link_clicked (url);
  // We're done.
  return true;
}


void InDialogHelp::html_link_clicked (const gchar * url)
{
  ustring myurl (url);
  // Erase "http://" part of the url.
  myurl.erase (0, 7);
  // See whether we've got an internal or external link.
  bool internal_link = false;
  if (myurl.length () > 17) {
    if (myurl.find ("51516") != string::npos) {
      ustring url2 (myurl);
      url2.erase (0, 16);
      if (url2.substr (0, 1) == "#") {
        internal_link = true;
        myurl = url2;
        myurl.erase (0, 1);
      }
    }
  }
  if (internal_link) {
    // Handle internal link.
    gtk_html_jump_to_anchor (GTK_HTML (htmlview), myurl.c_str());
  } else {
    // Show external url in the browser.
    htmlbrowser (myurl, true);
  }
}
