/*
** Copyright (C) 2003-2007 Teus Benschop.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 3 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#ifndef INCLUDED_IPC_H
#define INCLUDED_IPC_H


#include "libraries.h"
#include <gtk/gtkbutton.h>
#include <sys/socket.h>
#include <sys/un.h>


enum IPCSocketType {ipcstBibleditBin, ipcstBibleditBibletime, ipcstBibleditSubversion};

enum IPCCallType {ipcctBibleTimeReference,
                  ipcctBibleTimeShutdown,
                  ipcctBibleTimeConnected,
                  ipcctBibleTimeGetModules,
                  ipcctBibleTimeBibles,
                  ipcctBibleTimeCommentaries,
                  ipcctBibleTimeReloadModules,
                  ipcctBibleTimeSearchCommand,
                  ipcctBibleTimeSearchDone,
                  ipcctBibleTimeSearchResults,
                  ipcctSubversionJobDescription,
                  ipcctSubversionTaskDone,
                  ipcctSubversionShutdown,
                  ipcctSubversionReference,
                  ipcctSubversionReopenProject,
                  ipcctEnd};


class InterprocessCommunication
{
public:
  InterprocessCommunication (IPCSocketType name);
  ~InterprocessCommunication ();
  void send (IPCSocketType name, IPCCallType method, const vector<ustring>& payload);
  vector <ustring> receive (IPCSocketType name, IPCCallType method, const vector<ustring>& payload, int timeout = 5);
  vector<ustring> get_payload (IPCCallType method);
  void erase_payload (IPCCallType method);
  void methodcall_add_signal (IPCCallType method);
  GtkWidget * method_called_signal;
  IPCCallType method_called_type;
  void methodcall_remove_all_signals ();
private:
  struct sockaddr_un address;
  int sock, conn;
  socklen_t addrLength;
  ustring socketname (IPCSocketType socket);
  IPCSocketType myname;
  static void listener_start (gpointer data);
  void listener_main ();
  bool listener_run;
  bool listener_running;
  void log (const ustring& message, bool critical);
  map<IPCCallType, vector<ustring> > methodcalls;
  set<IPCCallType> signalling_methods;
};


#endif
