/*
** Copyright (C) 2003-2007 Teus Benschop.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 3 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "layout.h"
#include "gtkwrappers.h"
#include "settings.h"
#include "gui_features.h"


void screen_layout_tools_area_set (bool current_pos, 
                                   GtkWidget * parentleft, GtkWidget * parentright, 
                                   GtkWidget * childleft, GtkWidget * childright)
// Sets Tools Area left or right.
{
  extern Settings * settings;
  // Current and desired position are the same: bail out.
  if (current_pos == settings->genconfig.tools_area_left_get ())
    return;
  // Swap the left and right side.
  if (settings->genconfig.tools_area_left_get ()) {
    // Swapped
    gtk_widget_reparent (childleft, parentright);
    gtk_widget_reparent (childright, parentleft);
  } else {
    // Normal
    gtk_widget_reparent (childleft, parentleft);
    gtk_widget_reparent (childright, parentright);
  }
}


void screen_layout_vertical_pane_mirror (bool current_pos, GtkWidget * pane)
// Mirrors the divisions of the main vertical pane, if need be.
{
  extern Settings * settings;
  // Current and desired position are the same: bail out.
  if (current_pos == settings->genconfig.tools_area_left_get ())
    return;
  int position = gtk_paned_get_position (GTK_PANED (pane));
  position = settings->genconfig.window_width_get () - position;
  gtk_paned_set_position (GTK_PANED (pane), position);
}


void screen_layout_window_size_save (GtkWidget * window)
{
  extern Settings * settings;
  if (!settings->genconfig.window_maximized_get ()) {
    gint width, height, x, y;
    gtk_window_get_size (GTK_WINDOW (window), &width, &height);
    gtk_window_get_position (GTK_WINDOW (window), &x, &y);
    settings->genconfig.window_width_set (width);
    settings->genconfig.window_height_set (height);
    settings->genconfig.window_x_position_set (x);
    settings->genconfig.window_y_position_set (y);
  }     
}


void screen_layout_window_size_load (GtkWidget * window) 
{
  extern Settings * settings;
  gtk_window_resize (GTK_WINDOW (window), settings->genconfig.window_width_get (), settings->genconfig.window_height_get ());
  gtk_window_move (GTK_WINDOW (window), settings->genconfig.window_x_position_get (), settings->genconfig.window_y_position_get ());
}


void screen_layout_panes_position_save (GtkWidget * hpane, GtkWidget * editor, GtkWidget * references)
{
  GuiFeatures guifeatures (0);
  extern Settings * settings;
  if (guifeatures.project_notes ())
    settings->genconfig.hpane_position_set (gtk_paned_get_position (GTK_PANED (hpane)));
  settings->genconfig.vpane_editor_position_set (gtk_paned_get_position (GTK_PANED (editor)));
  settings->genconfig.vpane_quick_references_position_set (gtk_paned_get_position (GTK_PANED (references)));
}


void screen_layout_panes_position_load (GtkWidget * hpane, GtkWidget * editor, GtkWidget * references)
{
  extern Settings * settings;
  gtk_paned_set_position (GTK_PANED (hpane), settings->genconfig.hpane_position_get ());
  gtk_paned_set_position (GTK_PANED (editor), settings->genconfig.vpane_editor_position_get ());
  gtk_paned_set_position (GTK_PANED (references), settings->genconfig.vpane_quick_references_position_get ());
}
