/*
** Copyright (C) 2003-2007 Teus Benschop.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 3 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#ifndef INCLUDED_USFM_2_ABIWORD_H
#define INCLUDED_USFM_2_ABIWORD_H


#include "libraries.h"
#include "style.h"
#include <libxml/xmlwriter.h>


class Usfm2Abiword
{
public:
  Usfm2Abiword (const vector<ustring>& lines, vector<Style> * styles);
  ~Usfm2Abiword ();
  vector<ustring> output;
  bool success;
private:
  xmlBufferPtr buffer;
  xmlTextWriterPtr writer;
  void write_paragraph (ustring paragraph);
  unsigned int xid;
  void add_xid (ustring& line);
  void write_xid_property ();
  vector<Style> * mystyles;
  ustring font;
  void assemble_styles ();
  void assemble_style (Style * style, bool paragraph, bool plaintext, const ustring& font);
  void assemble_notes_styles ();
  void collect_paragraphs (const vector<ustring>& lines, vector<ustring>& paragraphs);
  void process_inline_text (ustring& paragraph);
  void place_styles_for_verse (ustring& paragraph, const ustring& verse_marker, const ustring& style_opener, const ustring& style_closer);
  void place_styles_for_inline_text (ustring& paragraph, const ustring& opener, const ustring& closer, const ustring& style_opener, const ustring& style_closer);
  void process_notes (ustring& paragraph);
  unsigned int footnoteid;
  void process_footnote (ustring& paragraph, const ustring& opener, const ustring& closer);
  unsigned int endnoteid;
  void process_endnote (ustring& paragraph, const ustring& opener, const ustring& closer);
};


#endif
