/*
** Copyright (C) 2003-2007 Teus Benschop.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 3 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "versification.h"
#include "utilities.h"
#include "bible.h"
#include "books.h"
#include "sqlite_reader.h"
#include "gwrappers.h"
#include "directories.h"
#include "versifications.h"


void versification_get_chapters_verses (const ustring& versification,
                                        unsigned int book,
                                        vector<unsigned int>& chapters,
                                        vector<unsigned int>& verses)
{
  extern Versifications * versifications;
  versifications->book2chaptersverses (versification, book, chapters, verses);
}


void versification_create_book_template (const ustring& versification, unsigned int book, vector<ustring>& booktemplate)
{
  // Clear template and add id and running header.
  booktemplate.clear();
  unsigned index = book;
  ustring id = books_id_to_paratext (index);
  booktemplate.push_back ("\\id " + id);
  booktemplate.push_back ("\\h " + book);
  booktemplate.push_back ("\\toc2 " + book);
  // Get the number of chapters in the book and verses in the chapter.
  vector<unsigned int> chapters;
  vector<unsigned int> verses;
  versification_get_chapters_verses (versification, book, chapters, verses);
  // Add \c and \v information for all chapters.
  for (unsigned int i = 0; i < chapters.size(); i++) {
    booktemplate.push_back ("\\c " + convert_to_string (chapters[i]));
    // Some paragraph ought to follow the chapter, so insert it here.
    booktemplate.push_back ("\\p ");
    for (unsigned int verse = 1; verse <= verses[i]; verse++) {
      booktemplate.push_back ("\\v " + convert_to_string (verse));
    }
  }
}


void versification_create_chapter_template (const ustring& versification, unsigned int book, unsigned int chapter, vector<ustring>& chaptertemplate)
{
  chaptertemplate.clear();
  vector<unsigned int> chapters;
  vector<unsigned int> verses;
  versification_get_chapters_verses (versification, book, chapters, verses);
  for (unsigned int i = 0; i < chapters.size(); i++) {
    if (chapters[i] == chapter) {
      chaptertemplate.push_back ("\\c " + convert_to_string (chapter));
      chaptertemplate.push_back ("\\p ");
      for (unsigned int verse = 1; verse <= verses[i]; verse++) {
        chaptertemplate.push_back ("\\v " + convert_to_string (verse));
      }
    }
  }
}


vector <unsigned int> versification_get_chapters (const ustring& versification, unsigned int book)
{
  vector<unsigned int> chapters;
  vector<unsigned int> verses;
  versification_get_chapters_verses (versification, book, chapters, verses);
  return chapters;
}


ustring versification_get_last_verse (const ustring& versification, unsigned int book, unsigned int chapter)
{
  ustring lastverse = "1";
  vector<unsigned int> chapters;
  vector<unsigned int> verses;
  versification_get_chapters_verses (versification, book, chapters, verses);
  for (unsigned int i = 0; i < chapters.size(); i++) {
    if (chapters[i] == chapter) {
      lastverse = convert_to_string (verses[i]);
    }
  }
  return lastverse;
}
